@@ -77,7 +77,8 @@ from typing import Collection, Iterable, Iterator, List, Tuple, Type, Union
 
 from appdirs import user_cache_dir
 
-from . import BaseCache, BaseStorage, get_valid_kwargs
+from .._utils import chunkify, get_valid_kwargs
+from . import BaseCache, BaseStorage
 
 MEMORY_URI = 'file::memory:?cache=shared'
 SQLITE_MAX_VARIABLE_NUMBER = 999
@@ -101,7 +102,7 @@ class SQLiteCache(BaseCache):
 
     def __init__(self, db_path: AnyPath = 'http_cache', **kwargs):
         super().__init__(**kwargs)
-        self.responses = SQLitePickleDict(db_path, table_name='responses', **kwargs)
+        self.responses: SQLiteDict = SQLitePickleDict(db_path, table_name='responses', **kwargs)
         self.redirects = SQLiteDict(db_path, table_name='redirects', **kwargs)
 
     @property
@@ -130,6 +131,10 @@ class SQLiteCache(BaseCache):
             self.responses.init_db()
             self.redirects.init_db()
 
+    def remove_expired_responses(self, *args, **kwargs):
+        with self.responses._lock, self.redirects._lock:
+            return super().remove_expired_responses(*args, **kwargs)
+
 
 class SQLiteDict(BaseStorage):
     """A dictionary-like interface for SQLite"""
@@ -145,14 +150,13 @@ class SQLiteDict(BaseStorage):
         **kwargs,
     ):
         super().__init__(**kwargs)
+        self._can_commit = True
+        self._local_context = threading.local()
+        self._lock = threading.RLock()
         self.connection_kwargs = get_valid_kwargs(sqlite_template, kwargs)
         self.db_path = _get_sqlite_cache_path(db_path, use_cache_dir, use_temp, use_memory)
         self.fast_save = fast_save
         self.table_name = table_name
-
-        self._lock = threading.RLock()
-        self._can_commit = True
-        self._local_context = threading.local()
         self.init_db()
 
     def init_db(self):
@@ -242,16 +246,17 @@ class SQLiteDict(BaseStorage):
         column = 'key' if keys else 'value'
         with self.connection(commit=True) as con:
             # Split into small enough chunks for SQLite to handle
-            for chunk in chunkify(keys or values):
+            for chunk in chunkify(keys or values, max_size=SQLITE_MAX_VARIABLE_NUMBER):
                 marks, args = _format_sequence(chunk)
                 statement = f'DELETE FROM {self.table_name} WHERE {column} IN ({marks})'
                 con.execute(statement, args)
 
     def clear(self):
-        with self.connection(commit=True) as con:
-            con.execute(f'DROP TABLE IF EXISTS {self.table_name}')
-        self.init_db()
-        self.vacuum()
+        with self._lock:
+            with self.connection(commit=True) as con:
+                con.execute(f'DROP TABLE IF EXISTS {self.table_name}')
+            self.init_db()
+            self.vacuum()
 
     def vacuum(self):
         with self.connection(commit=True) as con:
@@ -271,13 +276,6 @@ class SQLitePickleDict(SQLiteDict):
         return self.serializer.loads(super().__getitem__(key))
 
 
-def chunkify(iterable: Iterable, max_size=SQLITE_MAX_VARIABLE_NUMBER) -> Iterator[List]:
-    """Split an iterable into chunks of a max size"""
-    iterable = list(iterable)
-    for index in range(0, len(iterable), max_size):
-        yield iterable[index : index + max_size]
-
-
 def _format_sequence(values: Collection) -> Tuple[str, List]:
     """Get SQL parameter marks for a sequence-based query, and ensure value is a sequence"""
     if not isinstance(values, Iterable):
@@ -312,9 +310,15 @@ def get_cache_path(db_path: AnyPath, use_cache_dir: bool = False, use_temp: bool
     elif use_temp and not db_path.is_absolute():
         db_path = Path(gettempdir()) / db_path
 
-    # Expand relative and user paths (~/*), and make sure parent dirs exist
-    db_path = db_path.absolute().expanduser()
-    db_path.parent.mkdir(parents=True, exist_ok=True)
+    # Expand relative and user paths (~), make parent dir(s), and better error if parent is a file
+    db_path = db_path.expanduser().absolute()
+    try:
+        db_path.parent.mkdir(parents=True, exist_ok=True)
+    except FileExistsError:
+        raise FileExistsError(
+            f'Parent path exists and is not a directory: {db_path.parent}.'
+            'Please either delete the file or choose a different path.'
+        )
     return db_path
 
 
