@@ -16,19 +16,19 @@
 from contextlib import contextmanager
 from logging import getLogger
 from threading import RLock
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, Optional
+from typing import TYPE_CHECKING, Callable, Dict, Iterable, Optional
 
 from requests import PreparedRequest, Response
 from requests import Session as OriginalSession
 from requests.hooks import dispatch_hook
 from urllib3 import filepost
 
-from . import get_valid_kwargs
+from ._utils import get_valid_kwargs
 from .backends import BackendSpecifier, init_backend
-from .cache_control import CacheActions, ExpirationTime
-from .cache_keys import normalize_dict
+from .cache_control import CacheActions, ExpirationTime, get_expiration_seconds
 from .models import AnyResponse, CachedResponse, set_response_defaults
 
+__all__ = ['ALL_METHODS', 'CachedSession', 'CacheMixin']
 ALL_METHODS = ['GET', 'HEAD', 'OPTIONS', 'POST', 'PUT', 'PATCH', 'DELETE']
 FILTER_FN = Callable[[AnyResponse], bool]
 
@@ -66,8 +66,6 @@ class CacheMixin(MIXIN_BASE):
         self.filter_fn = filter_fn or (lambda r: True)
         self.stale_if_error = stale_if_error or kwargs.pop('old_data_on_error', False)
 
-        self.cache.name = cache_name  # Set to handle backend=<instance>
-        self._request_expire_after: ExpirationTime = None
         self._disabled = False
         self._lock = RLock()
 
@@ -75,13 +73,11 @@ class CacheMixin(MIXIN_BASE):
         session_kwargs = get_valid_kwargs(super().__init__, kwargs)
         super().__init__(**session_kwargs)  # type: ignore
 
-    def request(  # type: ignore  # Note: Session.request() doesn't have expire_after param
+    def request(  # type: ignore  # Note: An extra param (expire_after) is added here
         self,
         method: str,
         url: str,
-        params: Dict = None,
-        data: Any = None,
-        json: Dict = None,
+        *args,
         expire_after: ExpirationTime = None,
         **kwargs,
     ) -> AnyResponse:
@@ -95,8 +91,7 @@ class CacheMixin(MIXIN_BASE):
         Args:
             expire_after: Expiration time to set only for this request; see details below.
                 Overrides ``CachedSession.expire_after``. Accepts all the same values as
-                ``CachedSession.expire_after`` except for ``None``; use ``-1`` to disable expiration
-                on a per-request basis.
+                ``CachedSession.expire_after``. Use ``-1`` to disable expiration.
 
         Returns:
             Either a new or cached response
@@ -110,26 +105,30 @@ class CacheMixin(MIXIN_BASE):
         5. :py:meth:`.BaseCache.get_response`
         6. :py:meth:`requests.Session.send` (if not previously cached)
         7. :py:meth:`.BaseCache.save_response` (if not previously cached)
-
         """
-        with self.request_expire_after(expire_after), patch_form_boundary(**kwargs):
-            return super().request(
-                method,
-                url,
-                params=normalize_dict(params),
-                data=normalize_dict(data),
-                json=normalize_dict(json),
-                **kwargs,
-            )
+        # If present, set per-request expiration as a request header, to be handled in send()
+        if expire_after is not None:
+            kwargs.setdefault('headers', {})
+            kwargs['headers']['Cache-Control'] = f'max-age={get_expiration_seconds(expire_after)}'
 
-    def send(self, request: PreparedRequest, **kwargs) -> AnyResponse:
-        """Send a prepared request, with caching. See :py:meth:`.request` for notes on behavior."""
-        # Determine which actions to take based on request info, headers, and cache settings
+        with patch_form_boundary(**kwargs):
+            return super().request(method, url, *args, **kwargs)
+
+    def send(
+        self, request: PreparedRequest, expire_after: ExpirationTime = None, **kwargs
+    ) -> AnyResponse:
+        """Send a prepared request, with caching. See :py:meth:`.request` for notes on behavior, and
+        see :py:meth:`requests.Session.send` for parameters. Additional parameters:
+
+        Args:
+            expire_after: Expiration time to set only for this request
+        """
+        # Determine which actions to take based on request info and cache settings
         cache_key = self.cache.create_key(request, **kwargs)
         actions = CacheActions.from_request(
             cache_key=cache_key,
             request=request,
-            request_expire_after=self._request_expire_after,
+            request_expire_after=expire_after,
             session_expire_after=self.expire_after,
             urls_expire_after=self.urls_expire_after,
             cache_control=self.cache_control,
@@ -143,7 +142,7 @@ class CacheMixin(MIXIN_BASE):
             actions.update_from_cached_response(cached_response)
         is_expired = getattr(cached_response, 'is_expired', False)
 
-        # If the response is expired, missing, or the cache is disabled, then fetch a new response
+        # If the response is expired or missing, or the cache is disabled, then fetch a new response
         if cached_response is None:
             response = self._send_and_cache(request, actions, **kwargs)
         elif is_expired and self.stale_if_error:
@@ -153,7 +152,7 @@ class CacheMixin(MIXIN_BASE):
         else:
             response = cached_response
 
-        # If the request has been filtered out, delete the previously cached response if it exists
+        # If the request has been filtered out and was previously cached, delete it
         if not self.filter_fn(response):
             logger.debug(f'Deleting filtered response for URL: {response.url}')
             self.cache.delete(cache_key)
@@ -162,60 +161,6 @@ class CacheMixin(MIXIN_BASE):
         # Dispatch any hooks here, because they are removed before pickling
         return dispatch_hook('response', request.hooks, response, **kwargs)
 
-    def _send_and_cache(
-        self,
-        request: PreparedRequest,
-        actions: CacheActions,
-        cached_response: CachedResponse = None,
-        **kwargs,
-    ) -> AnyResponse:
-        """Send the request and cache the response, unless disabled by settings or headers.
-
-        If applicable, also add request headers to check if the remote resource has been modified.
-        If we get a 304 Not Modified response, return the expired cache item.
-        """
-        request.headers.update(actions.request_headers)
-        response = super().send(request, **kwargs)
-        actions.update_from_response(response)
-
-        if self._is_cacheable(response, actions):
-            self.cache.save_response(response, actions.cache_key, actions.expires)
-        elif cached_response and response.status_code == 304:
-            logger.debug(f'Response for URL {request.url} has not been modified; using cached response')
-            return cached_response
-        else:
-            logger.debug(f'Skipping cache write for URL: {request.url}')
-        return set_response_defaults(response, actions.cache_key)
-
-    def _resend(
-        self, request: PreparedRequest, actions: CacheActions, cached_response: CachedResponse, **kwargs
-    ) -> AnyResponse:
-        """Attempt to resend the request and cache the new response. If the request fails, delete
-        the expired cache item.
-        """
-        logger.debug('Expired response; attempting to re-send request')
-        try:
-            return self._send_and_cache(request, actions, cached_response, **kwargs)
-        except Exception:
-            self.cache.delete(actions.cache_key)
-            raise
-
-    def _resend_and_ignore(
-        self, request: PreparedRequest, actions: CacheActions, cached_response: CachedResponse, **kwargs
-    ) -> AnyResponse:
-        """Attempt to resend the request and cache the new response. If there are any errors, ignore
-        them and and return the expired cache item.
-        """
-        # Attempt to send the request and cache the new response
-        logger.debug('Expired response; attempting to re-send request')
-        try:
-            response = self._send_and_cache(request, actions, cached_response, **kwargs)
-            response.raise_for_status()
-            return response
-        except Exception:
-            logger.warning(f'Request for URL {request.url} failed; using cached response', exc_info=True)
-            return cached_response
-
     def _is_cacheable(self, response: Response, actions: CacheActions) -> bool:
         """Perform all checks needed to determine if the given response should be saved to the cache"""
         cache_criteria = {
@@ -228,6 +173,82 @@ class CacheMixin(MIXIN_BASE):
         logger.debug(f'Pre-cache checks for response from {response.url}: {cache_criteria}')
         return not any(cache_criteria.values())
 
+    def _send_and_cache(
+        self,
+        request: PreparedRequest,
+        actions: CacheActions,
+        cached_response: CachedResponse = None,
+        **kwargs,
+    ) -> AnyResponse:
+        """Send the request and cache the response, unless disabled by settings or headers.
+
+        If applicable, also add headers to make a conditional request. If we get a 304 Not Modified
+        response, return the stale cache item.
+        """
+        request.headers.update(actions.validation_headers)
+        response = super().send(request, **kwargs)
+        actions.update_from_response(response)
+
+        if self._is_cacheable(response, actions):
+            self.cache.save_response(response, actions.cache_key, actions.expires)
+        elif cached_response and response.status_code == 304:
+            return self._update_revalidated_response(actions, response, cached_response)
+        else:
+            logger.debug(f'Skipping cache write for URL: {request.url}')
+        return set_response_defaults(response, actions.cache_key)
+
+    def _resend(
+        self,
+        request: PreparedRequest,
+        actions: CacheActions,
+        cached_response: CachedResponse,
+        **kwargs,
+    ) -> AnyResponse:
+        """Attempt to resend the request and cache the new response. If the request fails, delete
+        the stale cache item.
+        """
+        logger.debug('Stale response; attempting to re-send request')
+        try:
+            return self._send_and_cache(request, actions, cached_response, **kwargs)
+        except Exception:
+            self.cache.delete(actions.cache_key)
+            raise
+
+    def _resend_and_ignore(
+        self,
+        request: PreparedRequest,
+        actions: CacheActions,
+        cached_response: CachedResponse,
+        **kwargs,
+    ) -> AnyResponse:
+        """Attempt to resend the request and cache the new response. If there are any errors, ignore
+        them and and return the stale cache item.
+        """
+        # Attempt to send the request and cache the new response
+        logger.debug('Stale response; attempting to re-send request')
+        try:
+            response = self._send_and_cache(request, actions, cached_response, **kwargs)
+            response.raise_for_status()
+            return response
+        except Exception:
+            logger.warning(
+                f'Request for URL {request.url} failed; using cached response', exc_info=True
+            )
+            return cached_response
+
+    def _update_revalidated_response(
+        self, actions: CacheActions, response: Response, cached_response: CachedResponse
+    ) -> CachedResponse:
+        """After revalidation, update the cached response's headers and reset its expiration"""
+        logger.debug(
+            f'Response for URL {response.request.url} has not been modified; updating and using cached response'
+        )
+        cached_response.headers.update(response.headers)
+        actions.update_from_response(cached_response)
+        cached_response.expires = actions.expires
+        self.cache.save_response(cached_response, actions.cache_key, actions.expires)
+        return cached_response
+
     @contextmanager
     def cache_disabled(self):
         """
@@ -251,16 +272,6 @@ class CacheMixin(MIXIN_BASE):
             finally:
                 self._disabled = False
 
-    @contextmanager
-    def request_expire_after(self, expire_after: ExpirationTime = None):
-        """Temporarily override ``expire_after`` for an individual request. This is needed to
-        persist the value between requests.Session.request() -> send()."""
-        # TODO: Is there a way to pass this via request kwargs -> PreparedRequest?
-        with self._lock:
-            self._request_expire_after = expire_after
-            yield
-            self._request_expire_after = None
-
     def remove_expired_responses(self, expire_after: ExpirationTime = None):
         """Remove expired responses from the cache, optionally with revalidation
 
@@ -314,8 +325,9 @@ class CachedSession(CacheMixin, OriginalSession):
 
 @contextmanager
 def patch_form_boundary(**request_kwargs):
-    """Patch the form boundary used to separate multipart uploads. ``requests`` does not provide a
-    way to pass a custom boundary to urllib3, so this just monkey-patches it instead.
+    """If the ``files`` param is present, patch the form boundary used to separate multipart
+    uploads. ``requests`` does not provide a way to pass a custom boundary to urllib3, so this just
+    monkey-patches it instead.
     """
     if request_kwargs.get('files'):
         original_boundary = filepost.choose_boundary
