@@ -1,15 +1,17 @@
 from unittest.mock import patch
 
 import pytest
+from gridfs import GridFS
+from gridfs.errors import CorruptGridFile, FileExists
 from pymongo import MongoClient
 
+from requests_cache._utils import get_valid_kwargs
 from requests_cache.backends import (
     GridFSCache,
     GridFSPickleDict,
     MongoCache,
     MongoDict,
     MongoPickleDict,
-    get_valid_kwargs,
 )
 from tests.conftest import fail_if_no_connection
 from tests.integration.base_cache_test import BaseCacheTest
@@ -65,6 +67,22 @@ class TestGridFSPickleDict(BaseStorageTest):
         GridFSPickleDict('test', host='http://0.0.0.0', port=1234, invalid_kwarg='???')
         mock_client.assert_called_with(host='http://0.0.0.0', port=1234)
 
+    def test_corrupt_file(self):
+        """A corrupted file should be handled and raise a KeyError instead"""
+        cache = self.init_cache()
+        cache['key'] = 'value'
+        with pytest.raises(KeyError), patch.object(GridFS, 'find_one', side_effect=CorruptGridFile):
+            cache['key']
+
+    def test_file_exists(self):
+        cache = self.init_cache()
+
+        # This write should just quiety fail
+        with patch.object(GridFS, 'put', side_effect=FileExists):
+            cache['key'] = 'value_1'
+
+        assert 'key' not in cache
+
 
 class TestGridFSCache(BaseCacheTest):
     backend_class = GridFSCache
