@@ -4,21 +4,19 @@ import json
 import time
 from collections import UserDict, defaultdict
 from datetime import datetime, timedelta
+from pathlib import Path
 from pickle import PickleError
 from unittest.mock import patch
+from urllib.parse import urlencode
 
 import pytest
 import requests
+from requests import Request, RequestException
 from requests.structures import CaseInsensitiveDict
 
 from requests_cache import ALL_METHODS, CachedResponse, CachedSession
-from requests_cache.backends import (
-    BACKEND_CLASSES,
-    BaseCache,
-    SQLiteDict,
-    SQLitePickleDict,
-    get_placeholder_class,
-)
+from requests_cache._utils import get_placeholder_class
+from requests_cache.backends import BACKEND_CLASSES, BaseCache, SQLiteDict, SQLitePickleDict
 from requests_cache.backends.base import DESERIALIZE_ERRORS
 from requests_cache.cache_keys import create_key
 from tests.conftest import (
@@ -38,6 +36,11 @@ def test_init_unregistered_backend():
         CachedSession(backend='nonexistent')
 
 
+def test_cache_path_expansion():
+    session = CachedSession('~', backend='filesystem')
+    assert session.cache.cache_dir == Path("~").expanduser()
+
+
 @patch.dict(BACKEND_CLASSES, {'mongo': get_placeholder_class()})
 def test_init_missing_backend_dependency():
     """Test that the correct error is thrown when a user does not have a dependency installed"""
@@ -55,9 +58,24 @@ def test_init_backend_instance():
     assert session.cache is backend
 
 
+def test_init_backend_instance__kwargs():
+    backend = MyCache()
+    session = CachedSession(
+        'test_cache',
+        backend=backend,
+        ignored_parameters=['foo'],
+        include_get_headers=True,
+    )
+
+    assert session.cache.cache_name == 'test_cache'
+    assert session.cache.ignored_parameters == ['foo']
+    assert session.cache.match_headers is True
+
+
 def test_init_backend_class():
-    session = CachedSession(backend=MyCache)
+    session = CachedSession('test_cache', backend=MyCache)
     assert isinstance(session.cache, MyCache)
+    assert session.cache.cache_name == 'test_cache'
 
 
 @pytest.mark.parametrize('method', ALL_METHODS)
@@ -107,6 +125,12 @@ def test_all_methods__ignored_parameters__redacted(field, method, mock_session):
     assert 'access_token' not in cached_response.request.body.decode('utf-8')
 
 
+def test_params_positional_arg(mock_session):
+    mock_session.request('GET', MOCKED_URL, {'param_1': 1})
+    response = mock_session.request('GET', MOCKED_URL, {'param_1': 1})
+    assert 'param_1=1' in response.url
+
+
 def test_https(mock_session):
     assert mock_session.get(MOCKED_URL_HTTPS, verify=True).from_cache is False
     assert mock_session.get(MOCKED_URL_HTTPS, verify=True).from_cache is True
@@ -136,12 +160,12 @@ def test_response_history(mock_session):
 
 def test_repr(mock_session):
     """Test session and cache string representations"""
-    mock_session.expire_after = 10.5
+    mock_session.expire_after = 11
     mock_session.cache.responses['key'] = 'value'
     mock_session.cache.redirects['key'] = 'value'
     mock_session.cache.redirects['key_2'] = 'value'
 
-    assert mock_session.cache.name in repr(mock_session) and '10.5' in repr(mock_session)
+    assert mock_session.cache.cache_name in repr(mock_session) and '11' in repr(mock_session)
     assert '2 redirects' in str(mock_session.cache) and '1 responses' in str(mock_session.cache)
 
 
@@ -278,15 +302,17 @@ def test_raw_data(method, mock_session):
     """POST and PUT requests with different data (raw) should be cached under different keys"""
     assert mock_session.request(method, MOCKED_URL, data='raw data').from_cache is False
     assert mock_session.request(method, MOCKED_URL, data='raw data').from_cache is True
-    assert mock_session.request(method, MOCKED_URL, data='new raw data').from_cache is False
+    assert (
+        mock_session.request(method, MOCKED_URL, data='{"data": "new raw data"}').from_cache
+        is False
+    )
 
 
-@pytest.mark.parametrize('mapping_class', [dict, UserDict, CaseInsensitiveDict])
 @pytest.mark.parametrize('field', ['params', 'data', 'json'])
-def test_normalize_params(field, mapping_class, mock_session):
-    """Test normalization with different combinations of data fields and dict-like classes"""
+def test_normalize_params(field, mock_session):
+    """Test normalization with different combinations of data fields"""
     params = {"a": "a", "b": ["1", "2", "3"], "c": "4"}
-    reversed_params = mapping_class(sorted(params.items(), reverse=True))
+    reversed_params = dict(sorted(params.items(), reverse=True))
 
     assert mock_session.get(MOCKED_URL, **{field: params}).from_cache is False
     assert mock_session.get(MOCKED_URL, **{field: params}).from_cache is True
@@ -296,25 +322,47 @@ def test_normalize_params(field, mapping_class, mock_session):
     assert mock_session.post(MOCKED_URL, **{field: {"a": "b"}}).from_cache is False
 
 
-@pytest.mark.parametrize('field', ['data', 'json'])
-def test_normalize_serialized_body(field, mock_session):
-    """Test normalization for serialized request body content"""
+@pytest.mark.parametrize('mapping_class', [dict, UserDict, CaseInsensitiveDict])
+def test_normalize_params__custom_dicts(mapping_class, mock_session):
+    """Test normalization with different dict-like classes"""
     params = {"a": "a", "b": ["1", "2", "3"], "c": "4"}
-    reversed_params = dict(sorted(params.items(), reverse=True))
+    params = mapping_class(params.items())
 
-    assert mock_session.post(MOCKED_URL, **{field: json.dumps(params)}).from_cache is False
-    assert mock_session.post(MOCKED_URL, **{field: json.dumps(params)}).from_cache is True
-    assert mock_session.post(MOCKED_URL, **{field: json.dumps(reversed_params)}).from_cache is True
+    assert mock_session.get(MOCKED_URL, params=params).from_cache is False
+    assert mock_session.get(MOCKED_URL, params=params).from_cache is True
+    assert mock_session.post(MOCKED_URL, params=params).from_cache is False
+    assert mock_session.post(MOCKED_URL, params=params).from_cache is True
 
 
-def test_normalize_non_json_body(mock_session):
+def test_normalize_params__serialized_body(mock_session):
+    """Test normalization for serialized request body content"""
+    headers = {'Content-Type': 'application/json'}
+    params = {"a": "a", "b": ["1", "2", "3"], "c": "4"}
+    sorted_params = json.dumps(params)
+    reversed_params = json.dumps(dict(sorted(params.items(), reverse=True)))
+
+    assert mock_session.post(MOCKED_URL, headers=headers, data=sorted_params).from_cache is False
+    assert mock_session.post(MOCKED_URL, headers=headers, data=sorted_params).from_cache is True
+    assert mock_session.post(MOCKED_URL, headers=headers, data=reversed_params).from_cache is True
+
+
+def test_normalize_params__urlencoded_body(mock_session):
+    headers = {'Content-Type': 'application/x-www-form-urlencoded'}
+    params = urlencode({"a": "a", "b": "!@#$%^&*()[]", "c": "4"})
+
+    assert mock_session.post(MOCKED_URL, headers=headers, data=params).from_cache is False
+    assert mock_session.post(MOCKED_URL, headers=headers, data=params).from_cache is True
+    assert mock_session.post(MOCKED_URL, headers=headers, data=params).from_cache is True
+
+
+def test_normalize_params__non_json_body(mock_session):
     """For serialized request body content that isn't in JSON format, no normalization is expected"""
     assert mock_session.post(MOCKED_URL, data=b'key_1=value_1,key_2=value_2').from_cache is False
     assert mock_session.post(MOCKED_URL, data=b'key_1=value_1,key_2=value_2').from_cache is True
     assert mock_session.post(MOCKED_URL, data=b'key_2=value_2,key_1=value_1').from_cache is False
 
 
-def test_normalize_url(mock_session):
+def test_normalize_params__url(mock_session):
     """Test URL variations that should all result in the same key"""
     urls = [
         'https://site.com?param_1=value_1&param_2=value_2',
@@ -402,7 +450,7 @@ def test_response_defaults(mock_session):
     response_1 = mock_session.get(MOCKED_URL)
     response_2 = mock_session.get(MOCKED_URL)
     response_3 = mock_session.get(MOCKED_URL)
-    cache_key = '71c046cdb0afaa62'
+    cache_key = 'd7fa9fb7317b7412'
 
     assert response_1.cache_key == cache_key
     assert response_1.created_at is None
@@ -422,21 +470,39 @@ def test_response_defaults(mock_session):
 def test_match_headers(mock_session):
     """With match_headers, requests with different headers should have different cache keys"""
     mock_session.cache.match_headers = True
-    headers_list = [{'Accept': 'text/json'}, {'Accept': 'text/xml'}, {'Accept': 'custom'}, None]
+    headers_list = [
+        {'Accept': 'application/json'},
+        {'Accept': 'text/xml'},
+        {'Accept': 'custom'},
+        None,
+    ]
     for headers in headers_list:
         assert mock_session.get(MOCKED_URL, headers=headers).from_cache is False
         assert mock_session.get(MOCKED_URL, headers=headers).from_cache is True
 
 
-def test_match_headers_normalize(mock_session):
+def test_match_headers__normalize(mock_session):
     """With match_headers, the same headers (in any order) should have the same cache key"""
     mock_session.cache.match_headers = True
-    headers = {'Accept': 'text/json', 'Custom': 'abc'}
-    reversed_headers = {'Custom': 'abc', 'Accept': 'text/json'}
+    headers = {'Accept': 'application/json', 'Custom': 'abc'}
+    reversed_headers = {'Custom': 'abc', 'Accept': 'application/json'}
     assert mock_session.get(MOCKED_URL, headers=headers).from_cache is False
     assert mock_session.get(MOCKED_URL, headers=reversed_headers).from_cache is True
 
 
+def test_match_headers__list(mock_session):
+    """match_headers can optionally be a list of specific headers to include"""
+    mock_session.cache.match_headers = ['Accept']
+    headers_1 = {'Accept': 'application/json', 'User-Agent': 'qutebrowser'}
+    headers_2 = {'Accept': 'application/json', 'User-Agent': 'Firefox'}
+    headers_3 = {'Accept': 'text/plain', 'User-Agent': 'qutebrowser'}
+
+    assert mock_session.get(MOCKED_URL, headers=headers_1).from_cache is False
+    assert mock_session.get(MOCKED_URL, headers=headers_1).from_cache is True
+    assert mock_session.get(MOCKED_URL, headers=headers_2).from_cache is True
+    assert mock_session.get(MOCKED_URL, headers=headers_3).from_cache is False
+
+
 def test_include_get_headers():
     """include_get_headers is aliased to match_headers for backwards-compatibility"""
     session = CachedSession(include_get_headers=True, backend='memory')
@@ -455,9 +521,9 @@ def test_expired_request_error(mock_session):
     """Without stale_if_error (default), if there is an error while re-fetching an expired
     response, the request should be re-raised and the expired item deleted"""
     mock_session.stale_if_error = False
-    mock_session.expire_after = 0.01
+    mock_session.expire_after = 1
     mock_session.get(MOCKED_URL)
-    time.sleep(0.01)
+    time.sleep(1)
 
     with patch.object(mock_session.cache, 'save_response', side_effect=ValueError):
         with pytest.raises(ValueError):
@@ -468,12 +534,12 @@ def test_expired_request_error(mock_session):
 def test_stale_if_error__exception(mock_session):
     """With stale_if_error, expect to get old cache data if there is an exception during a request"""
     mock_session.stale_if_error = True
-    mock_session.expire_after = 0.2
+    mock_session.expire_after = 1
 
     assert mock_session.get(MOCKED_URL).from_cache is False
     assert mock_session.get(MOCKED_URL).from_cache is True
-    time.sleep(0.2)
-    with patch.object(mock_session.cache, 'save_response', side_effect=ValueError):
+    time.sleep(1)
+    with patch.object(mock_session.cache, 'save_response', side_effect=RequestException):
         response = mock_session.get(MOCKED_URL)
         assert response.from_cache is True and response.is_expired is True
 
@@ -481,12 +547,12 @@ def test_stale_if_error__exception(mock_session):
 def test_stale_if_error__error_code(mock_session):
     """With stale_if_error, expect to get old cache data if a response has an error status code"""
     mock_session.stale_if_error = True
-    mock_session.expire_after = 0.2
+    mock_session.expire_after = 1
     mock_session.allowable_codes = (200, 404)
 
     assert mock_session.get(MOCKED_URL_404).from_cache is False
-    assert mock_session.get(MOCKED_URL_404).from_cache is True
-    time.sleep(0.2)
+
+    time.sleep(1)
     response = mock_session.get(MOCKED_URL_404)
     assert response.from_cache is True and response.is_expired is True
 
@@ -541,7 +607,9 @@ def test_do_not_cache(mock_session):
         (304, True, True, True),
     ],
 )
-def test_304_not_modified(response_code, cache_hit, cache_expired, expected_from_cache, mock_session):
+def test_304_not_modified(
+    response_code, cache_hit, cache_expired, expected_from_cache, mock_session
+):
     url = f'{MOCKED_URL}/endpoint_2'
     if cache_expired:
         mock_session.expire_after = datetime.now() - timedelta(1)
@@ -568,7 +636,9 @@ def test_url_allowlist(mock_session):
 
 def test_remove_expired_responses(mock_session):
     unexpired_url = f'{MOCKED_URL}?x=1'
-    mock_session.mock_adapter.register_uri('GET', unexpired_url, status_code=200, text='mock response')
+    mock_session.mock_adapter.register_uri(
+        'GET', unexpired_url, status_code=200, text='mock response'
+    )
     mock_session.expire_after = timedelta(seconds=0.2)
     mock_session.get(MOCKED_URL)
     mock_session.get(MOCKED_URL_JSON)
@@ -636,40 +706,64 @@ def test_remove_expired_responses__per_request(mock_session):
     mock_session.mock_adapter.register_uri('GET', second_url, status_code=200)
     mock_session.mock_adapter.register_uri('GET', third_url, status_code=200)
     mock_session.get(MOCKED_URL)
-    mock_session.get(second_url, expire_after=0.4)
-    mock_session.get(third_url, expire_after=0.8)
+    mock_session.get(second_url, expire_after=1)
+    mock_session.get(third_url, expire_after=2)
 
     # All 3 responses should still be cached
     mock_session.remove_expired_responses()
+    for response in mock_session.cache.responses.values():
+        print('Expires:', response.expires - datetime.utcnow() if response.expires else None)
     assert len(mock_session.cache.responses) == 3
 
-    # One should be expired after 0.4s, and another should be expired after 0.8s
-    time.sleep(0.4)
+    # One should be expired after 1s, and another should be expired after 2s
+    time.sleep(1)
     mock_session.remove_expired_responses()
     assert len(mock_session.cache.responses) == 2
-    time.sleep(0.4)
+    time.sleep(2)
     mock_session.remove_expired_responses()
     assert len(mock_session.cache.responses) == 1
 
 
-def test_per_request__expiration(mock_session):
-    """No per-session expiration is set, but then overridden with per-request expiration"""
+def test_per_request__enable_expiration(mock_session):
+    """No per-session expiration is set, but then overridden for a single request"""
     mock_session.expire_after = None
-    response = mock_session.get(MOCKED_URL, expire_after=0.01)
+    response = mock_session.get(MOCKED_URL, expire_after=1)
     assert response.from_cache is False
-    time.sleep(0.01)
+    assert mock_session.get(MOCKED_URL).from_cache is True
+
+    time.sleep(1)
+    response = mock_session.get(MOCKED_URL)
+    assert response.from_cache is False
+
+
+def test_per_request__disable_expiration(mock_session):
+    """A per-session expiration is set, but then disabled for a single request"""
+    mock_session.expire_after = 60
+    response = mock_session.get(MOCKED_URL, expire_after=-1)
+    response = mock_session.get(MOCKED_URL, expire_after=-1)
+    assert response.from_cache is True
+    assert response.expires is None
+
+
+def test_per_request__prepared_request(mock_session):
+    """The same should work for PreparedRequests with CachedSession.send()"""
+    mock_session.expire_after = None
+    request = Request(method='GET', url=MOCKED_URL, headers={}, data=None).prepare()
+    response = mock_session.send(request, expire_after=1)
+    assert response.from_cache is False
+    assert mock_session.send(request).from_cache is True
+
+    time.sleep(1)
     response = mock_session.get(MOCKED_URL)
     assert response.from_cache is False
 
 
 def test_per_request__no_expiration(mock_session):
     """A per-session expiration is set, but then overridden with no per-request expiration"""
-    mock_session.expire_after = 0.01
+    mock_session.expire_after = 1
     response = mock_session.get(MOCKED_URL, expire_after=-1)
     assert response.from_cache is False
-    time.sleep(0.01)
-    response = mock_session.get(MOCKED_URL)
-    assert response.from_cache is True
+    assert response.expires is None
 
 
 def test_unpickle_errors(mock_session):
